﻿#region Using

using System;
using RemoteCamera.Processing;
using RemoteCamera.ViewModels;
using Windows.UI.Core;
using Windows.UI.Xaml.Controls;
using RemoteCamera.AzureHelpers;

#endregion

namespace RemoteCamera
{    
    public sealed partial class MainPage : Page
    {
        #region Fields

        private RemoteCameraViewModel remoteCameraViewModel;

        #endregion

        #region Constructor

        public MainPage()
        {
            InitializeComponent();

            remoteCameraViewModel = new RemoteCameraViewModel(CaptureElementPreview);

            remoteCameraViewModel.ImageProcessor.ProcessingDone += ImageProcessor_ProcessingDone;
            remoteCameraViewModel.CloudHelper.UpdateCameraPreviewCommandReceived += CloudHelper_UpdateCameraPreviewCommandReceived;
        }

        #endregion

        #region Methods

        private void ImageProcessor_ProcessingDone(object sender, ImageProcessorEventArgs e)
        {
            // Update display through dispatcher
            DisplayBrightness(e.Brightness);

            // Send telemetry
            if (remoteCameraViewModel.IsTelemetryActive)
            {
                remoteCameraViewModel.CloudHelper.SendBrightness(e.Brightness);
            }
        }

        private async void DisplayBrightness(byte brightness)
        {
            if (Dispatcher.HasThreadAccess)
            {
                remoteCameraViewModel.Brightness = brightness;
            }
            else
            {
                await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                {
                    DisplayBrightness(brightness);
                });
            }
        }

        private async void CloudHelper_UpdateCameraPreviewCommandReceived(object sender, UpdateCameraPreviewCommandEventArgs e)
        {
            if (Dispatcher.HasThreadAccess)
            {
                if (e.IsPreviewActive)
                {
                    await remoteCameraViewModel.PreviewStart();
                }
                else
                {
                    await remoteCameraViewModel.PreviewStop();
                }
            }
            else
            {
                await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
                {
                    CloudHelper_UpdateCameraPreviewCommandReceived(sender, e);
                });
            }
        }

        #endregion
    }
}
